import ts from 'typescript';
import { dumpNode } from "./dumpNode.js";
import { nameToString } from "./util.js";
/** @internal */
export class ImportIdentifierCollection {
    checker;
    _length = 0;
    idRegistry = new Map();
    unusedIds = new Set();
    constructor(checker) {
        this.checker = checker;
    }
    get length() {
        return this._length;
    }
    add(node) {
        if (this._addNamespaceImport(node)) {
            return;
        }
        this._addNamedImport(node);
        this._addDefaultImport(node);
    }
    /** import def from */
    _addDefaultImport(node) {
        const name = node.importClause?.name;
        if (!name) {
            return false;
        }
        this.pushItem(node, name, 'default');
        return true;
    }
    /** import {a as b, c} from */
    _addNamedImport(node) {
        if (!node.importClause?.namedBindings || !ts.isNamedImports(node.importClause.namedBindings)) {
            return false;
        }
        const namedBindings = node.importClause.namedBindings;
        for (const item of namedBindings.elements) {
            this.pushItem(node, item.name, nameToString(item.propertyName || item.name));
        }
        return true;
    }
    /** import * as all from  */
    _addNamespaceImport(node) {
        if (!node.importClause?.namedBindings || !ts.isNamespaceImport(node.importClause.namedBindings)) {
            return false;
        }
        const namedBindings = node.importClause.namedBindings;
        this.pushItem(node, namedBindings.name, '*');
        return true;
    }
    pushItem(node, identifier, stringifyName) {
        const symbol = this.checker.getSymbolAtLocation(identifier);
        if (symbol) {
            this._length++;
            // printMyDiagnostic(node, '%s imported as %s', stringifyName, identifier.getText());
            this.idRegistry.set(identifier, { node, identifier, stringifyName });
            this.unusedIds.add(identifier);
        }
        else {
            // printMyDiagnostic(node, 'imported identifier "%s" has no symbol', node.getText());
        }
    }
    findDeclaration(decl) {
        let ret;
        const checkSame = (id) => {
            if (id && this.unusedIds.has(id)) {
                return id;
            }
            return undefined;
        };
        if (ts.isImportSpecifier(decl)) {
            // import {a} || import { a as b }
            return checkSame(decl.name);
        }
        else if (ts.isNamespaceImport(decl)) {
            // import * as a
            return checkSame(decl.name);
        }
        else if (ts.isImportClause(decl)) {
            // import def
            return checkSame(decl.name);
        }
        return ret;
    }
    found(id) {
        const info = this.idRegistry.get(id);
        if (!info) {
            dumpNode([...this.idRegistry.keys(), id]);
            throw new Error('program error.');
        }
        // console.log('!!!! %s (%s) used, is a value', info.stringifyName, info.node.getText());
        this.unusedIds.delete(id);
        return info;
    }
    listNotFound() {
        const ret = [];
        for (const id of this.unusedIds.values()) {
            ret.push(this.idRegistry.get(id));
        }
        return ret;
    }
}
//# sourceMappingURL=importIds.helper.js.map